/*---------------------------------------------------------------------------*/
// Project Name : CpuInfo.dll
//      Licence : GNU GPL
//       Author : hiyohiyo
//         Mail : hiyohiyo@crystalmark.info
//          Web : http://crystalmark.info/
/*---------------------------------------------------------------------------*/

#include "CpuInfo.h"
#include "CpuInfoID.h"
#include "msrnt.h"

#include <stdio.h>

BOOL ReadMSR(ULONG ulECX,ULONG* pulEAX,ULONG* pulEDX);
int CallRing0(PVOID pvRing0FuncAddr,ULONG ulECX,ULONG* ulEAX,ULONG* ulEDX);
void Ring0ReadMSR();
static BOOL IsNT();

static BOOL IsNT()
{
	OSVERSIONINFO OSVersionInfo;
	OSVersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&OSVersionInfo);
	
	return OSVersionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT;
}

CCpuInfo::CCpuInfo(DWORD mask)
{
	if(IsNT()){mask = (DWORD)SetThreadAffinityMask(GetCurrentThread(),mask);}

	InitData();

	if(CheckUseCPUID()){
		FillCpuInfo();
		FillCacheInfo();
		if(MaxCPUIDEx >= 0x80000004){FillNameString();}
		if(MaxCPUIDEx >= 0x80000007){FlagAMDMobile = CheckAMDMobile();}
		if(FlagBrand == TMx86){FillTMx86Ex();}
		SetCPUName();
		if(FlagRDTSC){Clock = CheckClock() / 1000000.0;}
		if(FlagRDMSR){
			SetUpRDMSR();
			SetCPUName();// Multiplier 񂪕Kv CPU o
		}
	}

	if(IsNT()){SetThreadAffinityMask(GetCurrentThread(),mask);}
}

int CCpuInfo::CheckAMDMobile()
{
	DWORD dwEDX;
	_asm{
		mov eax,0x80000007
		cpuid
		mov dwEDX,edx
	}
	return (dwEDX >> 1) & 0x1;
}

int CCpuInfo::SetUpRDMSR()
{
	if(	FlagBrand == INTEL && Family == 6 ){
		ReadMSR(0x2A,&MsrEAX1,&MsrEDX1);
		if(Model == 8){// Coppermine ONLY!
			ReadMSR(0x17,&MsrEAX2,&MsrEDX2);
			PlatformID = ( MsrEDX2 >> 18 ) & 0x7;
		}
		if(Model == 9){// Banias for Multiplier
			ReadMSR(0x17,&MsrEAX2,&MsrEDX2);
		}
		// char str[256];wsprintf(str,"%0X",PlatformID);MessageBox(NULL,str,str,MB_OK);
	}else if( FlagMultiTable == Samuel2 || FlagMultiTable == Ezra || FlagMultiTable == Cyrix3 ){
		ReadMSR(0x2A,&MsrEAX1,&MsrEDX1);
	}else if(FlagBrand == INTEL && Family == 0xF && FamilyEx == 0x0){ // Pentium4
		ReadMSR(0x2A,&MsrEAX1,&MsrEDX1);
		ReadMSR(0x2C,&MsrEAX2,&MsrEDX2);
	}else if(FlagBrand == AMD && Family == 0xF && Model == 5){// Opteron
	//	2003/7/3 pc̕񍐂ɂꎞx~...
		// AMD 26094.pdf
	//	ReadMSR(0xC0010041,&MsrEAX1,&MsrEDX1);
	}else if(FlagBrand == AMD && Family == 5 && Model >= 8){// K6-2 & K6-III
		ReadMSR(0xC0000087,&MsrEAX1,&MsrEDX1);
	}else if(FlagBrand == AMD && Family == 6){// Athlon
		ReadMSR(0xC0010015,&MsrEAX1,&MsrEDX1);
		if(Model <= 2){ // Athlon K7/K75
			ReadMSR(0xC0010011,&MsrEAX2,&MsrEDX2);
		}else 
		// C0010042 // Power Now!
		// C001001B // CLK_CTL
		if( FlagAMDMobile ){
			ReadMSR(0xC0010042,&MsrEAX2,&MsrEDX2);
		}
	}else if(FlagBrand == IDT && Family == 5 && Model >= 8){ // WinChip 2/3
		ReadMSR(0x10A,&MsrEAX1,&MsrEDX1);
		if( (Model == 8 && 7 <= Stepping && Stepping <= 9) // WinChip 2A
		||	(Model == 9) // WinChip 3
		){
			ReadMSR(0x147,&MsrEAX2,&MsrEDX2);
		}
	}
	SetCacheSpeed();
	if( MsrEAX1 != 0 || MsrEAX2 != 0){
		if(FlagBrand == TMx86){
			ClockOri = TmNominalClock;
		}else{
			SetCPUFSB();
		}
	}
	if(ClockOri > 0.0){
		OverClock = Clock / ClockOri * 100.0 - 100.0;
	}
	return 0;
}

int CCpuInfo::SetCacheSpeed()
{
	if(CacheL2 <= 0){return -1;} // L2 Cache Ȃꍇ͗e͂Ȃ|C
	if(FlagBrand == AMD && Family == 6 && Model <= 2){ // Athlon K7/K75
		DWORD CacheInfo = (MsrEAX2>>4) & 0x3;

		switch(CacheInfo){
		case 0:strcpy(CacheSpeedS,"1/2");CacheSpeed = 0.50;break;
		case 1:strcpy(CacheSpeedS,"2/5");CacheSpeed = 0.40;break;
		case 2:strcpy(CacheSpeedS,"1/3");CacheSpeed = 0.33;break;
		}
	}else if(FlagBrand == INTEL && FlagHalfSpeedCache){
		strcpy(CacheSpeedS,"1/2");
		CacheSpeed = 0.50;
	}else{
		strcpy(CacheSpeedS,"Full");
		CacheSpeed = 1.00;
	}
	return 0;
}

BOOL ReadMSR(ULONG ulECX,ULONG* pulEAX,ULONG* pulEDX)
{
	if(IsNT()){	
		if(! _ReadMSR(ulECX,pulEAX,pulEDX) ){
			return FALSE;
		}else{
			return TRUE;
		}
	}else{
		CallRing0(Ring0ReadMSR,ulECX,pulEAX,pulEDX);
		return TRUE;
	}
}

CCpuInfo::~CCpuInfo()
{
}

int CCpuInfo::GetString( DWORD ID , char* s)
{
	char str[16];

	switch(ID)
	{
// String
	case	CPU_NAME_SYS_INFO:		strcpy(s,NameSysInfo);	break;
	case	CPU_FSB_MODE:			strcpy(s,FSBMode);		break;
	case	CPU_VENDOR_STRING:		strcpy(s,VendorString);	break;
	case	CPU_VENDOR_NAME:		strcpy(s,VendorName);	break;
	case	CPU_NAME_STRING:		strcpy(s,NameString);	break;
	case	CPU_PLATFORM_NAME:		strcpy(s,PlatformName);	break;
	case	CPU_TYPE_NAME:			strcpy(s,TypeName);		break;
	case	CPU_FMS:				strcpy(s,FMS);			break;
	case	CPU_TM_NAME_STRING:		strcpy(s,TmNameString);	break;
	case	CPU_CACHE_SPEED_STR:	strcpy(s,CacheSpeedS);	break;
	case	CPU_NAME:				strcpy(s,Name);			break;
	case	CPU_CODE_NAME:			strcpy(s,CodeName);		break;
	case	CPU_FULL_NAME:			strcpy(s,FullName);		break;
	case	CPU_PROCESS_RULE:		strcpy(s,ProcessRule);	break;

	case	CPU_CACHE_L1ITU_WAYS:	strcpy(s,L1ITUWaysS);	break;
	case	CPU_CACHE_L1ITU_LINES:	strcpy(s,L1ITULinesS);	break;
	case	CPU_CACHE_L1D_WAYS:		strcpy(s,L1DWaysS);		break;
	case	CPU_CACHE_L1D_LINES:	strcpy(s,L1DLinesS);	break;
	case	CPU_CACHE_L2_WAYS:		strcpy(s,L2WaysS);		break;
	case	CPU_CACHE_L2_LINES:		strcpy(s,L2LinesS);		break;
	case	CPU_CACHE_L3_WAYS:		strcpy(s,L3WaysS);		break;
	case	CPU_CACHE_L3_LINES:		strcpy(s,L3LinesS);		break;

// double
	// vG[ //
	case	CPU_CLOCK:				sprintf(str,"%4.2f",Clock);			strcpy(s,str);break;
	case	CPU_SYSTEM_CLOCK:		sprintf(str,"%4.2f",SystemClock);	strcpy(s,str);break;
	case	CPU_SYSTEM_BUS:			sprintf(str,"%4.2f",SystemBus);		strcpy(s,str);break;
	case	CPU_MULTIPLIER:			sprintf(str,"%4.2f",Multiplier);	strcpy(s,str);break;
	case	CPU_CLOCK_ORI:			sprintf(str,"%4.2f",ClockOri);		strcpy(s,str);break;
	case	CPU_SYSTEM_CLOCK_ORI:	sprintf(str,"%4.2f",SystemClockOri);strcpy(s,str);break;
	case	CPU_SYSTEM_BUS_ORI:		sprintf(str,"%4.2f",SystemBusOri);	strcpy(s,str);break;
	case	CPU_CACHE_SPEED:		sprintf(str,"%4.2f",CacheSpeed);	strcpy(s,str);break;
	case	CPU_OVER_CLOCK:			sprintf(str,"%4.2f",OverClock);		strcpy(s,str);break;
	case	CPU_MULTIPLIER_ORI:		sprintf(str,"%4.2f",MultiplierOri);	strcpy(s,str);break;
	case	CPU_CLOCK_UPDATE:		if(FlagRDTSC){Clock = CheckClock() / 1000000.0;}
									if(FlagRDMSR){SetUpRDMSR();}
									sprintf(str,"%4.2f",Clock);
									strcpy(s,str);
									break;
	default:
		strcpy(s,"");
		return -1;
	}
	return 0;
}

int CCpuInfo::GetData( DWORD ID , DWORD* d)
{
	switch(ID)
	{
	case	CPU_NUMBER:				*d = Number;		break;
	case	CPU_FAMILY:				*d = Family;		break;
	case	CPU_MODEL:				*d = Model;			break;
	case	CPU_STEPPING:			*d = Stepping;		break;
	case	CPU_FAMILY_EX:			*d = FamilyEx;		break;
	case	CPU_MODEL_EX:			*d = ModelEx;		break;
	case	CPU_BRAND_ID:			*d = BrandID;		break;
	case	CPU_FEATURE:			*d = Feature; 		break;
	case	CPU_FEATURE_ECX:		*d = FeatureEcx;	break;
	case	CPU_FEATURE_EX:			*d = FeatureEx;		break;
	case	CPU_CACHE_L1I:			*d = CacheL1I;		break;
	case	CPU_CACHE_L1T:			*d = CacheL1T;		break;
	case	CPU_CACHE_L1U:			*d = CacheL1U;		break;
	case	CPU_CACHE_L1D:			*d = CacheL1D;		break;
	case	CPU_CACHE_L2:			*d = CacheL2;		break;
	case	CPU_CACHE_L3:			*d = CacheL3;		break;
	case	CPU_HYPER_THREAD_NUM:	*d = HyperThreadNum;break;
	case	CPU_TYPE:				*d = Type;			break;
	case	CPU_RDMSR_EAX_1:		*d = MsrEAX1;		break;
	case	CPU_RDMSR_EAX_2:		*d = MsrEAX2;		break;
	case	CPU_FSB_MULTIPLIER:		*d = FSBMultiplier;	break;
	case	CPU_VERSION:			*d = Version;		break;
	case	CPU_VERSION_EX:			*d = VersionEx;		break;
	case	CPU_FLAG_BRAND:			*d = FlagBrand;		break;
	case	CPU_APIC:				*d = Apic;			break;

	case	CPU_FLAG_MMX:			*d = FlagMMX;		break;
	case	CPU_FLAG_MMX_EX:		*d = FlagMMXEx;		break;
	case	CPU_FLAG_SSE:			*d = FlagSSE;		break;
	case	CPU_FLAG_SSE2:			*d = FlagSSE2;		break;
	case	CPU_FLAG_SSE3:			*d = FlagSSE3;		break;
	case	CPU_FLAG_3DNOW:			*d = Flag3DNow;		break;
	case	CPU_FLAG_3DNOW_EX:		*d = Flag3DNowEx;	break;
	case	CPU_FLAG_HYPER_THREAD:	*d = FlagHT;		break;
	case	CPU_FLAG_AA64:			*d = FlagAA64;		break;
	case	CPU_FLAG_IA64:			*d = FlagIA64;		break;

	case	CPU_CACHE_L1ITU_WAYS:	*d = L1ITUWays;		break;
	case	CPU_CACHE_L1ITU_LINES:	*d = L1ITULines;	break;
	case	CPU_CACHE_L1D_WAYS:		*d = L1DWays;		break;
	case	CPU_CACHE_L1D_LINES:	*d = L1DLines;		break;
	case	CPU_CACHE_L2_WAYS:		*d = L2Ways;		break;
	case	CPU_CACHE_L2_LINES:		*d = L2Lines;		break;
	case	CPU_CACHE_L3_WAYS:		*d = L3Ways;		break;
	case	CPU_CACHE_L3_LINES:		*d = L3Lines;		break;

	case	CPU_RDMSR_EDX_1:		*d = MsrEDX1;		break;
	case	CPU_RDMSR_EDX_2:		*d = MsrEDX2;		break;
	case	CPU_TM_CLOCK:			*d = (DWORD)TmClock;break;
	case	CPU_TM_NOMINAL_CLOCK:	*d = (DWORD)TmNominalClock;break;

	default:
		*d = -1;
		return -1;
	}
	return 0;
}

void CCpuInfo::SetCPUName()
{
	if(FlagBrand == INTEL){
		SetCPUNameIntel();
	}else if(FlagBrand == AMD){
		SetCPUNameAMD();
	}else if(FlagBrand == TMx86){
		SetCPUNameTMx86();
	}else if(FlagBrand == CYRIX || FlagBrand == IDT){
		SetCPUNameVIA();
	}else{
		SetCPUNameOthers();
	}

// CPUName  CodeName m
	// Code Name 
	if(strlen(NameSysInfo) != 0 && strchr( NameSysInfo,'(' ) ){
		char str[65];
		char* token;
		strcpy(str,NameSysInfo);
		token = strtok( str , "(" );	strcpy(Name,str);
		token = strtok( NULL , ")" );	strcpy(CodeName,token);
		Name[strlen(Name) - 1] = NULL;
	}else{	// Code Name Ȃ
		strcpy(Name,NameSysInfo);
	}

	sprintf(FullName,"%s %s",VendorName,Name);
}

// Ƃ肠̎ //
// Brand ID ƒ`ĂΊyȂ̂ɁEEEB
void CCpuInfo::SetCPUNameIntel()
{
	int F = Family,M = Model,S = Stepping,FE = FamilyEx;
	char *n,*p,*r;
	n = "";
	p = "";
	r = "";

	if(0){}

// Brand ID ɂ鎎

//	else if(BrandID == 0xE){ n = "Xeon (Foster)";}
//	else if(BrandID == 0xB){ n = "Xeon (Prestonia)";FlagMultiTable=Northwood;}
//	else if(BrandID == 0x9){ n = "Pentium 4 (Northwood)";FlagMultiTable=Northwood;}
//	else if(BrandID == 0x8){ n = "Pentium 4 (Willamette)";FlagMultiTable=Willamette;}
	else if(BrandID == 0x7){
		n = "Mobile Celeron (Tualatin)";
		p = "Mobile Module";
		r = "0.13";
		FlagMultiTable=Tualatin;
		FlagClockModulation=1;
	}else if(BrandID == 0x6){
		n = "Mobile Pentium III (Tualatin)";
		p = "Mobile Module";
		r = "0.13";
		FlagMultiTable=Tualatin;
		FlagClockModulation=1;
	}
//	else if(BrandID == 0x4){ n = "Pentium III (Tualatin)";}
	else if(BrandID == 0x3 && M == 0xB && S >= 1){
		n = "Celeron (Tualatin)";
		p = "Socket 370";
		r = "0.13";
		FlagMultiTable=Tualatin;
	}
//	else if(BrandID == 0x3){ n = "Pentium III Xeon (Tanner)";FlagMultiTable=Coppermine;}
//	else if(BrandID == 0x2){ n = "Pentium III (Coppermine)";FlagMultiTable=Coppermine;}
//	else if(BrandID == 0x1){ n = "Celeron (Coppermine-128K)";FlagMultiTable=Coppermine;}

// Intel McKinley (IA-64)
	else if(F == 0xF && FE > 0x01){ n = "Itanium 3 (Madison)";}
	else if(F == 0xF && FE == 0x01){ n = "Itanium 2 (McKinley)";}

// Pentium 4
	else if(F == 0xF && M == 3){
		n = "Pentium 4 HT (Prescott)";
		r = "0.09";
		FlagMultiTable=Northwood;
	}else if(F == 0xF && M == 2 && BrandID == 0xB){
		n = "Xeon (Prestonia)";
		p = "Socket 603/604";
		r = "0.13";
		FlagMultiTable=Northwood;
	}else if(F == 0xF && M == 2 && BrandID == 0xE){
		n = "Mobile Pentium 4 (Northwood)";
		p = "Mobile Module";
		r = "0.13";
		FlagMultiTable=Northwood;
		FlagClockModulation=1;
	}else if(F == 0xF && M == 2 && HyperThreadNum >= 2){
		n = "Pentium 4 HT (Northwood)";
		r = "0.13";
		p = "Socket 478";
		FlagMultiTable=Northwood;
	}else if(F == 0xF && M == 2 && CacheL2 == 512){
		n = "Pentium 4 (Northwood)";
		p = "Socket 478";
		r = "0.13";
		FlagMultiTable=Northwood;
	}else if(F == 0xF && M == 2 && CacheL2 < 512 && strstr(NameString,"Mobile") ){
		n = "Mobile Celeron (Northwood-256K)";
		p = "Mobile Module";
		r = "0.13";
		FlagMultiTable=Northwood;
	}else if(F == 0xF && M == 2 && CacheL2 < 512){
		n = "Celeron (Northwood-128K)";
		p = "Socket 478";
		r = "0.13";
		FlagMultiTable=Northwood;
	}else if(F == 0xF && M <= 1 && BrandID == 0xE){
		n = "Xeon (Foster)";
		p = "Socket 603";
		r = "0.18";
		FlagMultiTable=Willamette;
	}else if(F == 0xF && M == 1 && S == 3){
		n = "Celeron (Willamette-128K)";
		p = "Socket 478";
		r = "0.18";
		FlagMultiTable=Willamette;
	}else if(F == 0xF && M == 1){
		n = "Pentium 4 (Willamette)";
		p = "Socket 478";
		r = "0.18";
		FlagMultiTable=Willamette;
	}else if(F == 0xF && M == 0){
		n = "Pentium 4 (Willamette)";
		p = "Socket 423";
		r = "0.18";
		FlagMultiTable=Willamette;
	}

// Merced
	else if(F == 0x7){
		n = "Itanium (Merced)";
	}

// Pentium III
	else if(F == 0x6 && M == 0xC){
		n = "Pentium M (Dothan)";
		p = "Mobile Module";
		r = "0.09";
		FlagMultiTable=Banias;
		FlagClockModulation=1;
	}else if(F == 0x6 && M == 0xB && CacheL2 == 512){
		n = "Pentium III-S (Tualatin)";
		p = "Socket 370";
		r = "0.13";
		FlagMultiTable=Tualatin;
	}else if(F == 0x6 && M == 0xB && BrandID == 0x4){
		n = "Pentium III (Tualatin)";
		p = "Socket 370";
		r = "0.13";
		FlagMultiTable=Tualatin;
	}else if(F == 0x6 && M == 0xB){
		n = "Celeron (Tualatin)";
		p = "Socket 370";
		r = "0.13";
		FlagMultiTable=Tualatin;
	}else if(F == 0x6 && M == 0xA){
		n = "Pentium III Xeon (Cascades)";
		r = "0.18";
		FlagMultiTable=Coppermine;
	}

// Banias
	else if(F == 0x6 && M == 0x9 && CacheL2 == 1024){
		n = "Pentium M (Banias)";
		p = "Mobile Module";
		r = "0.13";
		FlagMultiTable=Banias;
		FlagClockModulation=1;
	}else if(F == 0x6 && M == 0x9 && CacheL2 == 512){
		n = "Mobile Celeron (Banias)";
		p = "Mobile Module";
		r = "0.13";
		FlagMultiTable=Banias;
	}

	else if(F == 0x6 && M == 0x8 && CacheL2 > 256){
		n = "Pentium III Xeon (Cascades)";
		r = "0.18";
		FlagMultiTable=Coppermine;
	}else if(F == 0x6 && M == 0x8 && CacheL2 == 256 && PlatformID % 2){
		n = "Mobile Pentium III (Coppermine)";
		p = "Mobile Module";
		r = "0.18";
		FlagMultiTable=Coppermine;
		FlagClockModulation=1;
	}else if(F == 0x6 && M == 0x8 && CacheL2 == 256){
		n = "Pentium III (Coppermine)";
		p = "Socket 370";
		r = "0.18";
		FlagMultiTable=Coppermine;
	}else if(F == 0x6 && M == 0x8 && CacheL2 <= 128 && PlatformID % 2){
		n = "Mobile Celeron (Coppermine-128K)";
		p = "Mobile Module";
		r = "0.18";
		FlagMultiTable=Coppermine;
	}else if(F == 0x6 && M == 0x8 && CacheL2 <= 128){
		n = "Celeron (Coppermine-128K)";
		p = "Socket 370";
		r = "0.18";
		FlagMultiTable=Coppermine;
	}

	else if(F == 0x6 && M == 0x7 && CacheL2 > 512){
		n = "Pentium III Xeon (Tanner)";
		p = "Slot 2";
		r = "0.25";
		FlagMultiTable=P6;
	}else if(F == 0x6 && M == 0x7){
		n = "Pentium III (Katmai)";
		p = "Slot 1";
		r = "0.25";
		FlagMultiTable=P6;
		FlagHalfSpeedCache=TRUE;
	}

// Pentium II
	else if(F == 0x6 && M == 0x6 && (S == 0xA || S == 0xD ) && CacheL2 == 256){
		n = "Mobile Pentium II (Dixon)";
		p = "Mobile Module";
		r = "0.25";
		FlagMultiTable=P6;
	}else if(F == 0x6 && M == 0x6 && (S == 0xA || S == 0xD ) && CacheL2 == 128){
		n = "Mobile Celeron (Dixon-128K)";
		p = "Mobile Module";
		r = "0.25";
		FlagMultiTable=P6;
	}else if(F == 0x6 && M == 0x6){
		n = "Celeron (Mendocino)";
		p = "Socket 370";
		r = "0.25";
		FlagMultiTable=P6;
	}else if(F == 0x6 && M == 0x5 && CacheL2 > 512){
		n = "Pentium II Xeon (Deschutes)";
		p = "Slot 2";
		r = "0.25";
		FlagMultiTable=P6;
	}else if(F == 0x6 && M == 0x5 && CacheL2 == 512 && Type == 0x1){
		n = "Pentium II OverDrive";
		r = "0.25";
		FlagMultiTable=P6;
		FlagHalfSpeedCache=TRUE;
	}else if(F == 0x6 && M == 0x5 && CacheL2 == 512){
		n = "Pentium II (Deschutes)";
		p = "Slot 1";
		r = "0.25";
		FlagMultiTable=P6;
		FlagHalfSpeedCache=TRUE;
	}else if(F == 0x6 && M == 0x5 && CacheL2 == 0){
		n = "Celeron (Covinton)";
		p = "Slot 1";
		r = "0.25";
		FlagMultiTable=P6;
	}

	else if(F == 0x6 && M == 0x4){//?
		n = "OverDrive";
	}

	else if(F == 0x6 && M == 0x3 && Type == 0x1){
		n = "Pentium MMX OverDrive";
		r = "0.25";
		FlagMultiTable=P6;
	}else if(F == 0x6 && M == 0x3 && Type == 0x0){
		n = "Pentium II (Klamath)";
		p = "Slot 1";
		r = "0.28";
		FlagMultiTable=P6;
		FlagHalfSpeedCache=TRUE;
	}

// Pentium Pro
	else if(F == 0x6 && M <= 0x2){
		n = "Pentium Pro (P6)";
		p = "Socket 8";
		r = "0.35/0.50";
		FlagMultiTable=P6;
	}

// Pentium OverDrive
	else if(F == 0x5 && M == 0x4 && Type == 0x1){
		n = "Pentium MMX OverDrive";
		CacheL1I=16;
		CacheL1D=16;
	}else if(F == 0x5 && M == 0x2 && Type == 0x1){
		n = "Pentium OverDrive";
		CacheL1I=8;
		CacheL1D=8;
	}else if(F == 0x5 && M == 0x1 && Type == 0x1){
		n = "Pentium OverDrive";
		CacheL1I=8;
		CacheL1D=8;
	}

// Pentium 
	else if(F == 0x5 && M == 0x8){
		n = "Pentium MMX (P55)";
		r = "0.25";
		CacheL1I=16;
		CacheL1D=16;
	}else if(F == 0x5 && M == 0x7){
		n = "Pentium (P54C)";
		r = "0.35";
		CacheL1I=8;
		CacheL1D=8;
	}else if(F == 0x5 && M == 0x6){//??
		n = "Pentium OverDrive";
		CacheL1I=8;
		CacheL1D=8;
	}else if(F == 0x5 && M == 0x5){//??
		n = "Pentium OverDrive";
		CacheL1I=8;
		CacheL1D=8;
	}else if(F == 0x5 && M == 0x4){
		n = "Pentium MMX (P55C)";
		r = "0.28";
		CacheL1I=16;
		CacheL1D=16;
	}else if(F == 0x5 && M == 0x3){
		n = "Pentium OverDrive (P24T)";
		r = "0.35";
		CacheL1I=8;
		CacheL1D=8;
	}else if(F == 0x5 && M == 0x2){
		n = "Pentium (P54C)";
		r = "0.50";
		CacheL1I=8;
		CacheL1D=8;
	}else if(F == 0x5 && M <= 0x1){
		n = "Pentium (P5)";
		r = "0.80";
		CacheL1I=8;
		CacheL1D=8;
	}

// 486
	else if(F == 0x4 && M == 0x9){
		n = "486DX4 WB";
		CacheL1U=8;
	}else if(F == 0x4 && M == 0x8){
		n = "486DX4";
		CacheL1U=8;
	}else if(F == 0x4 && M == 0x7){
		n = "486DX2 WB";
		CacheL1U=8;
	}else if(F == 0x4 && M == 0x5){
		n = "486SX2";
		CacheL1U=8;
	}else if(F == 0x4 && M == 0x4){
		n = "486SL";
		CacheL1U=8;
	}else if(F == 0x4 && M == 0x3){
		n = "486DX2";
		CacheL1U=8;
	}else if(F == 0x4 && M == 0x2){
		n = "486SX";
		CacheL1U=8;
	}else if(F == 0x4 && M <= 0x1){
		n = "486DX";
		CacheL1U=8;
	}

// 386
	else if(F == 0x3){
		n = "386";
	}else{
		n = "Unknown CPU";
	}

	strcpy(NameSysInfo,n);
	strcpy(PlatformName,p);
	strcpy(ProcessRule,r);
}

void CCpuInfo::SetCPUNameAMD()
{
	int F = Family,M = Model,S = Stepping;
	int FlagAMDMP,FlagAMDXP;
	char *n,*p,*r;
	n = "";
	p = "";
	r = "";

	FlagAMDMP = ( strstr(NameString,"MP") != NULL );// || (FeatureEx >> 19) & 0x1 );
	FlagAMDXP = ( strstr(NameString,"XP") != NULL );

	if(0){}
// Hammer
	else if(F == 0xF && M == 5){
		n = "Opteron (SledgeHammer)";
		p = "Socket 940";
		r = "0.13";
		FlagMultiTable=Opteron;
	}else if(F == 0xF && M == 4){
		n = "Athlon 64 (ClawHammer)";
		p = "Socket 754";
		r = "0.13";
		FlagMultiTable=Opteron;
	}else if(F == 0xF){
		n = "Athlon 64 Series";
		FlagMultiTable=Opteron;
		r = "0.13";
	}

// Athlon
	else if(F == 0x6 && M == 0xA && FlagAMDMobile ){
		n = "Mobile Athlon XP (Barton)";
		p = "Socket A";
		r = "0.13";
		FlagMultiTable=MobileAthlon;
		FlagClockModulation=1;
	}else if(F == 0x6 && M == 0xA && CacheL2 == 512 && FlagAMDXP ){
		n = "Athlon XP (Barton)";
		p = "Socket A";
		r = "0.13";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M == 0xA && CacheL2 == 512 && FlagAMDMP ){
		n = "Athlon MP (Barton)";
		p = "Socket A";
		r = "0.13";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M == 0xA && CacheL2 == 256 && FlagAMDXP ){
		n = "Athlon FX (Thorton)";
		p = "Socket A";
		r = "0.13";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M == 0xA){
		n = "Athlon XP/MP (Barton)";
		p = "Socket A";
		r = "0.13";
		FlagMultiTable=Athlon;
	}

	else if(F == 0x6 && M == 0x8 && FlagAMDMobile ){
		n = "Mobile Athlon XP (Thoroughbred)";
		p = "Socket A";
		r = "0.13";
		FlagMultiTable=MobileAthlon;
		FlagClockModulation=1;
	}else if(F == 0x6 && M == 0x8 && CacheL2 >= 256 && FlagAMDXP ){
		n = "Athlon XP (Thoroughbred)";
		p = "Socket A";
		r = "0.13";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M == 0x8 && CacheL2 >= 256 && FlagAMDMP ){
		n = "Athlon MP (Thoroughbred)";
		p = "Socket A";
		r = "0.13";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M == 0x8 && CacheL2 >= 256){
		n = "Athlon XP/MP (Thoroughbred)";
		p = "Socket A";
		r = "0.13";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M == 0x8 && CacheL2 == 64){
		n = "Duron (Applebred)";
		p = "Socket A";
		r = "0.13";
		FlagMultiTable=Athlon;
	}

	else if(F == 0x6 && M == 0x7 && FlagAMDMobile){
		n = "Mobile Duron (Morgan)";
		p = "Socket A";
		r = "0.18";
		FlagMultiTable=MobileAthlon;
	}else if(F == 0x6 && M == 0x7 ){
		n = "Duron (Morgan)";
		p = "Socket A";
		r = "0.18";
		FlagMultiTable=Athlon;
	}
	
	else if(F == 0x6 && M == 0x6 && FlagAMDMobile && CacheL2 == 64){
		n = "Mobile Duron (Palomino)";
		p = "Socket A";
		r = "0.18";
		FlagMultiTable=MobileAthlon;
		FlagClockModulation=1;
	}else if(F == 0x6 && M == 0x6 && FlagAMDMobile ){
		n = "Mobile Athlon 4 (Palomino)";
		p = "Socket A";
		r = "0.18";
		FlagMultiTable=MobileAthlon;
		FlagClockModulation=1;
	}else if(F == 0x6 && M == 0x6 && FlagAMDMP ){
		n = "Athlon MP (Palomino)";
		p = "Socket A";
		r = "0.18";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M == 0x6 && FlagAMDXP ){
		n = "Athlon XP (Palomino)";
		p = "Socket A";
		r = "0.18";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M == 0x6 && CacheL2 == 64){
		n = "Duron (Morgan)";
		p = "Socket A";
		r = "0.18";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M == 0x6){
		n = "Athlon XP/MP (Palomino)";
		p = "Socket A";
		r = "0.18";
		FlagMultiTable=Athlon;
	}

	else if(F == 0x6 && M == 0x4){
		n = "Athlon (Thunderbird)";
		p = "Slot A / Socket A";
		r = "0.18";
		FlagMultiTable=Athlon;
	}

	else if(F == 0x6 && M == 0x3 && strstr(NameString,"Mobile") ){
		n = "Mobile Duron (Spitfire)";
		p = "Socket A";
		r = "0.18";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M == 0x3){
		n = "Duron (Spitfire)";
		p = "Socket A";
		r = "0.18";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M == 0x2){
		n = "Athlon (K75)";
		p = "Slot A";
		r = "0.18";
		FlagMultiTable=Athlon;
	}else if(F == 0x6 && M <= 0x1){
		n = "Athlon (K7)";
		p = "Slot A";
		r = "0.25";
		FlagMultiTable=Athlon;
	}

// K6 or K5(Am5x86) 
	else if(F == 0x5 && M == 0xD && CacheL2 == 256){
		n = "K6-III+";
		p = "Socket 7 / Mobile";
		r = "0.18";
		FlagMultiTable=K6;
	}else if(F == 0x5 && M == 0xD && CacheL2 == 128){
		n = "K6-2+";
		p = "Socket 7 / Mobile";
		r = "0.18";
		FlagMultiTable=K6;
	}else if(F == 0x5 && M == 0x9){
		n = "K6-III (Sharptooth)";
		p = "Socket 7";
		r = "0.25";
		FlagMultiTable=K6;
	}else if(F == 0x5 && M == 0x8){
		n = "K6-2 (K6-3D)";
		p = "Socket 7";
		r = "0.25";
		FlagMultiTable=K6;
	}else if(F == 0x5 && M == 0x7){
		n = "K6";
		p = "Socket 7";
		r = "0.25";
	}else if(F == 0x5 && M == 0x6){
		n = "K6";
		p = "Socket 7";
		r = "0.30";
	}else if(F == 0x5 && 0x1 <= M && M <= 0x3){
		n = "K5";
		p = "Socket 5/7";
		r = "0.35";
		CacheL1I=16;
		CacheL1D=8;
	}else if(F == 0x5 && M == 0x0){
		n = "K5";
		p = "Socket 5/7";
		r = "0.35/0.50";
		CacheL1I=16;
		CacheL1D=8;
	}

// 486 or 5x86
	else if(F == 0x4 && M == 0xF){
		n = "Am5x86";
		p = "Socket 3";
		CacheL1U=16;
	}else if(F == 0x4 && M == 0xE){
		n = "Am5x86";
		p = "Socket 3";
		CacheL1U=16;
	}else if(F == 0x4 && M == 0x9){
		n = "Am5x86";
		p = "Socket 3";
		CacheL1U=16;
	}else if(F == 0x4 && M == 0x8){
		n = "Am486DX4";
		p = "Socket 3";
		CacheL1U=8;
	}else if(F == 0x4 && M == 0x7){
		n = "Am486DX2";
		p = "Socket 3";
		CacheL1U=8;
	}else if(F == 0x4 && M == 0x3){
		n = "Am486DX2";
		p = "Socket 3";
		CacheL1U=8;
	}

	else{ n = "Unknown CPU";}

	strcpy(NameSysInfo,n);
	strcpy(PlatformName,p);
	strcpy(ProcessRule,r);
}

void CCpuInfo::SetCPUNameTMx86()
{
	int F = Family,M = Model,S = Stepping;
	char *n,*p,*r;
	n = "";
//	p = "";
	r = "";
	p = "Mobile Module";

	if(0){}
	else if(F == 0x5 && M == 0x4 && strstr(NameString,"3120") != NULL ){
		n = "Crusoe TM3120";
		r = "0.22";
		FlagClockModulation=1;
	}else if(F == 0x5 && M == 0x4 && strstr(NameString,"3200") != NULL ){
		n = "Crusoe TM3200";
		r = "0.22";
		FlagClockModulation=1;
	}else if(F == 0x5 && M == 0x4 && strstr(NameString,"5400") != NULL ){
		n = "Crusoe TM5400";
		r = "0.18";
		FlagClockModulation=1;
	}else if(F == 0x5 && M == 0x4 && strstr(NameString,"5600") != NULL ){
		n = "Crusoe TM5600";
		r = "0.18";
		FlagClockModulation=1;
	}else if(F == 0x5 && M == 0x4 && strstr(NameString,"5500") != NULL ){
		n = "Crusoe TM5500";
		r = "0.13";
		FlagClockModulation=1;
	}else if(F == 0x5 && M == 0x4 && strstr(NameString,"5800") != NULL ){
		n = "Crusoe TM5800";
		r = "0.13";
		FlagClockModulation=1;
	}else{
		n = "Unknown CPU";
	}

	strcpy(NameSysInfo,n);
	strcpy(PlatformName,p);
	strcpy(ProcessRule,r);
}


void CCpuInfo::SetCPUNameVIA()
{
	int F = Family,M = Model,S = Stepping;
	char *n,*p,*r;
	n = "";
	p = "";
	r = "";

	BOOL VIA = FALSE;

	if(0){}
	else if(F == 0x6 && M == 0x9){
		n = "C3 Nehemiah";
		p = "Socket 370";
		r = "0.13";
		FlagMultiTable=Ezra;
		VIA = TRUE;
	}else if(F == 0x6 && M == 0x8){
		n = "C3 Ezra";
		p = "Socket 370";
		r = "0.13";
		FlagMultiTable=Ezra;
		VIA = TRUE;
	}else if(F == 0x6 && M == 0x7 && S >= 8){
		n = "C3 Ezra";
		p = "Socket 370";
		r = "0.13";
		FlagMultiTable=Ezra;
		VIA = TRUE;
	}else if(F == 0x6 && M == 0x7 && S < 8){
		n = "C3 Samuel 2";
		p = "Socket 370";
		r = "0.15";
		FlagMultiTable=Samuel2;
		VIA = TRUE;
	}else if(F == 0x6 && M == 0x6){
		n = "Cyrix III (Samuel)";
		p = "Socket 370";
		r = "0.18";
		FlagMultiTable=Cyrix3;
		VIA = TRUE;
	}else if(F == 0x6 && M == 0x5){
		n = "Cyrix III (Joshua)";
		p = "Socket 370";
		r = "0.18";
		FlagMultiTable=Cyrix3;
		VIA = TRUE;
	}

	else if(F == 0x6 && M == 0x0){
		n = "6x86MX (M II)";
		p = "Socket 7";
		r = "0.35";
		CacheL1U = 64;
	}
	
	else if(F == 0x5 && M == 0x9 && FlagBrand == IDT ){
		n = "WinChip 3";
		p = "Socket 7";
		r = "0.25";
	}else if(F == 0x5 && M == 0x8 && FlagBrand == IDT ){
		n = "WinChip 2";
		p = "Socket 7";
		r = "0.35";
	}else if(F == 0x5 && M == 0x4 && FlagBrand == IDT ){
		n = "WinChip C6";
		p = "Socket 7";
		CacheL1I=32;
		r = "0.35";
		CacheL1D=32;
	}else if(F == 0x5 && M == 0x4 && FlagBrand == CYRIX ){
		n = "MediaGX";
		CacheL1U=16;
	}else if(F == 0x5 && M == 0x2 && FlagBrand == CYRIX ){
		n = "6x86 (M I)";
		p = "Socket 5/7";
		r = "0.35/0.65";
		CacheL1U=16;
	}

	else{ 
		n = "Unknown CPU";
	}

	strcpy(NameSysInfo,n);
	strcpy(PlatformName,p);
	strcpy(ProcessRule,r);

	if(VIA){
		strcpy(VendorName,"VIA");
	}
}

void CCpuInfo::SetCPUNameOthers()
{
	int F = Family,M = Model,S = Stepping;
	char *n,*p,*r;
	n = "";
	p = "";
	r = "";

	if(0){}
// NexGen
	else if(F == 0x5 && M == 0x0 && FlagBrand == NEXGEN ){
		n = "Nx586";
		r = "0.44/0.50";
		CacheL1I=16;
		CacheL1D=16;
	}

// Rise
	else if(F == 0x5 && M == 0x0 && FlagBrand == RISE ){
		n = "mP6 (iDragon 0.25)";
		p = "Socket 7";
		r = "0.25";
		CacheL1I=8;
		CacheL1D=8;
	}else if(F == 0x5 && M == 0x2 && FlagBrand == RISE ){
		n = "mP6 (iDragon 0.18)";
		p = "Socket 7";
		r = "0.18";
		CacheL1I=8;
		CacheL1D=8;
	}else if(F == 0x5 && M == 0x8 && FlagBrand == RISE ){
		n = "mP6 (iDragon II 0.25)";
		p = "Socket 7";
		r = "0.25";
		CacheL1I=8;
		CacheL1D=8;
	}else if(F == 0x5 && M == 0x9 && FlagBrand == RISE ){
		n = "mP6 (iDragon II 0.18)";
		p = "Socket 7";
		r = "0.18";
		CacheL1I=8;
		CacheL1D=8;
	}

	else{
		if( strcmp(VendorString,"Compaq FX!32") == 0 ){ n = "Alpha";}
		if( strcmp(VendorString,"ConnectixCPU") == 0 ){ n = "PowerPC";}
		
		// Alpha & PowerPC Check //
		SYSTEM_INFO si;
		GetSystemInfo(&si);
		switch(si.wProcessorArchitecture)
		{
		case PROCESSOR_ARCHITECTURE_MIPS:
			switch(si.wProcessorLevel){
			case 0004:n = "MIPS R4000";break;
			default:n = "MIPS";break;
			}
			break;
		case PROCESSOR_ARCHITECTURE_ALPHA:
			switch(si.wProcessorLevel){
			case 21064:n = "Alpha 21064";break;
			case 21066:n = "Alpha 21066";break;
			case 21164:n = "Alpha 21164";break;
			default:n = "Alpha";break;
			}
			break;
		case PROCESSOR_ARCHITECTURE_PPC:
			switch(si.wProcessorLevel){
			case 1:	n = "PowerPC 601";break;
			case 3:	n = "PowerPC 603";break;
			case 4:	n = "PowerPC 604";break;
			case 6:	n = "PowerPC 603+";break;
			case 9:	n = "PowerPC 604+";break;
			case 20:n = "PowerPC 620";break;
			default:n = "PowerPC";break;
			}
			break;
		default:
			n = "Unknown CPU";
			break;
		}
	}
	strcpy(NameSysInfo,n);
	strcpy(PlatformName,p);
	strcpy(ProcessRule,r);
}


//////////////////////////////////////////////////////////////////////
// CPU 𒲍
//////////////////////////////////////////////////////////////////////
int CCpuInfo::FillCpuInfo()
{
	DWORD dwEAX,dwEBX,dwECX,dwEDX;

// VendorName
	char vendor[13];
	vendor[12] = '\0';
	_asm{
		mov eax,0x0
		cpuid
		mov dwEAX , eax
		mov dword ptr [vendor] , ebx	
		mov dword ptr [vendor+4] , edx
		mov dword ptr [vendor+8] , ecx
	}
	MaxCPUID = dwEAX;
	sprintf(VendorString,vendor);

	if( strcmp(VendorString,"GenuineIntel") == 0 ){FlagBrand=INTEL;	sprintf(VendorName,"Intel");}
	if( strcmp(VendorString,"AuthenticAMD") == 0 ){FlagBrand=AMD;	sprintf(VendorName,"AMD");}
	if( strcmp(VendorString,"GenuineTMx86") == 0 ){FlagBrand=TMx86;	sprintf(VendorName,"Transmeta");}
	if( strcmp(VendorString,"CyrixInstead") == 0 ){FlagBrand=CYRIX;	sprintf(VendorName,"Cyrix");}
	if( strcmp(VendorString,"CentaurHauls") == 0 ){FlagBrand=IDT;	sprintf(VendorName,"IDT");}
	if( strcmp(VendorString,"RiseRiseRise") == 0 ){FlagBrand=RISE;	sprintf(VendorName,"Rise");}
	if( strcmp(VendorString,"NexGenDriven") == 0 ){FlagBrand=NEXGEN;sprintf(VendorName,"NexGen");}
	if( strcmp(VendorString,"Compaq FX32!") == 0 ){FlagBrand=COMPAQ;sprintf(VendorName,"Compaq");}

// Extended CPUID
	_asm{
		mov eax,0x80000000
		cpuid
		mov dwEAX , eax
	}
	MaxCPUIDEx = dwEAX;

// FeatureEx
	if(MaxCPUIDEx > 0x80000000 ){
		_asm{
			mov eax,0x80000001
			cpuid
			mov dwEAX , eax
			mov dwEDX , edx
		}
		if(FlagBrand != INTEL){
			VersionEx = dwEAX;
			FeatureEx = dwEDX;
		}
	}

	FlagMMXEx	= (FeatureEx >> 22) & 0x1;
	FlagAA64	= (FeatureEx >> 29) & 0x1;
	Flag3DNow	= (FeatureEx >> 31) & 0x1;
	Flag3DNowEx = (FeatureEx >> 30) & 0x1;

// Family / Model / Step
	_asm
	{
		mov eax,1
		cpuid
		mov dwEAX,eax
		mov dwEBX,ebx
		mov dwECX,ecx
		mov dwEDX,edx
	}

	Version = dwEAX;
	Feature = dwEDX;
	FeatureEcx = dwECX;
	Family	= (dwEAX>>8) & 0xF;
	Model	= (dwEAX>>4) & 0xF;
	Stepping= dwEAX & 0xF;
	Type    = (dwEAX>>12) & 0x3;

	if( FlagBrand == INTEL && Family == 0xF ){
		Apic = (dwEBX>>24) & 0xFF;
	}

	if(FlagBrand == INTEL){
		BrandID = dwEBX & 0xFF;
	}
	switch(Type)
	{
	case 0:	strcpy(TypeName,"Original OEM processor");	break;
	case 1:	strcpy(TypeName,"OverDrive processor");		break;
	case 2:	strcpy(TypeName,"Dual processor");			break;
	default:strcpy(TypeName,"Reserved");				break;
	}

	FlagMMX = (Feature>>23) & 0x1;
	FlagSSE = (Feature>>25) & 0x1;
	FlagSSE2 = (Feature>>26) & 0x1;
	FlagSSE3 = (FeatureEcx) & 0x1;
	FlagHT = (Feature>>28) & 0x1;
	FlagIA64 = (Feature>>30) & 0x1;

	if( FlagHT ){ // EnableHyperThreading
		HyperThreadNum = (dwEBX>>16) & 0xFF;
	}else{
		HyperThreadNum = 0;
	}
	if(HyperThreadNum <= 1){FlagHT = FALSE;}


	if(Family == 0xF){FamilyEx = (VersionEx>>20) & 0xFF;}
	if(Model == 0xF){ModelEx = (VersionEx>>16) & 0xF;}

// FSBMode //

	if(FlagBrand == INTEL && Family == 0xF && FamilyEx == 0x0){
		FSBMultiplier = 4;
		strcpy(FSBMode,"QDR");
	}else if(FlagBrand == INTEL && Family == 0x6 && ( Model == 0x9 || Model == 0xC ) ){//Pentium M
		FSBMultiplier = 4;
		strcpy(FSBMode,"QDR");
	}else if(FlagBrand == AMD && Family == 0xF){
		FSBMultiplier = 0;
		strcpy(FSBMode,"---");
	}else if(FlagBrand == AMD && Family >= 0x6){
		FSBMultiplier = 2;
		strcpy(FSBMode,"DDR");
	}else{
		FSBMultiplier = 1;
		strcpy(FSBMode,"SDR");
	}

	sprintf(FMS,"%X%X%X",Family,Model,Stepping);
	FlagRDTSC = (Feature>>4)&0x1;
	FlagRDMSR = (Feature>>5)&0x1;

	return 0;
}
//////////////////////////////////////////////////////////////////////
// NbNg̑
//////////////////////////////////////////////////////////////////////
double CCpuInfo::CheckClock()
{
	//xNbNg//
	//|Cg
	//ELARGE_INTEGER Ȃǂ64rbgp̂gȂB
	//EQPC mɓ삵Ȃꍇ GetTickCount() ŉ䖝
	//EQPC ܂Ƃɓ삵Ȃ}V̓}`fBA^C}[_mȂ̂...

	DWORD dwStart,dwEnd;					// GetTickCount()
	DWORD dwStartH,dwStartL,dwEndH,dwEndL;	// QPC
	DWORD TickCount;
	ULARGE_INTEGER lRdtscS,lRdtscE;
	LARGE_INTEGER lPre,lStart,lEnd,lFrequency;

// ב
	volatile int i=0;

	TickCount = GetTickCount();
	if(FlagClockModulation){
		while(GetTickCount() - TickCount < SLEEP_TIME){
			i++;
		}
	}

	QueryPerformanceCounter(&lPre);
	QueryPerformanceCounter(&lStart);
	_asm{
		rdtsc
		mov dwStartH , edx
		mov dwStartL , eax
	}
	dwStart = GetTickCount();
	
	// Sleep API Ƃ܂ɕsN邽
	TickCount = dwStart;
	while(GetTickCount() - TickCount < SLEEP_TIME){
		i++;
	}

	dwEnd = GetTickCount();
	_asm{
		rdtsc
		mov dwEndH , edx
		mov dwEndL , eax
	}
	QueryPerformanceCounter(&lEnd);
	QueryPerformanceFrequency(&lFrequency);
	lRdtscS.HighPart = dwStartH;
	lRdtscS.LowPart  = dwStartL;
	lRdtscE.HighPart = dwEndH;
	lRdtscE.LowPart  = dwEndL;

	double totalTime;
	double totalTimeQPC = (double)( (lEnd.QuadPart - lStart.QuadPart) - (lStart.QuadPart - lPre.QuadPart) ) / (double)(lFrequency.QuadPart);// bPʂɕϊ
	double totalTimeWT  = (double)(dwEnd - dwStart) / 1000.0;// bPʂɕϊ

	const double ERROR_RATE = (double)SLEEP_TIME/1000.0/10;
	if(totalTimeWT - ERROR_RATE < totalTimeQPC && totalTimeQPC < totalTimeWT + ERROR_RATE){
		totalTime = totalTimeQPC;
		FlagQPC = TRUE;
	}else{
		totalTime = totalTimeWT;
		FlagQPC = FALSE;
	}
	return (double)( (__int64)(lRdtscE.QuadPart - lRdtscS.QuadPart) / totalTime );

}

void CCpuInfo::SetCPUFSB()
{
	DWORD mul,bus;
	DWORD BusTable[8] = {   6666666, 13333333, 10000000, 16666666 , 20000000 , 0, 0, 0 };

// Debug
/*
	FlagBrand = AMD;
	FlagMultiTable = MobileAthlon;
	Model = 7;
	MsrEAX2 = 0x000E0401;

	FlagBrand = INTEL;
	FlagMultiTable = Banias;
	MsrEAX1 = 0x43480000;MsrEDX1 = 0xD00086AB;
	MsrEAX2 = 0x00000000;MsrEDX2 = 0x00140000;
*/

// DataSheet Q //
	static BYTE MultiTableEzra[16]  =	{	10,  6,  8, 20, 11,  7,  9, 19,
		     								18, 14, 16, 12, 24, 15, 17, 13};
	static BYTE MultiTableSamuel2[16] =	{	10,  0,  8, 20, 11,  7,  9, 22,
											18, 14, 16, 12, 24, 15, 26, 13};
	static BYTE MultiTableCryix3[16] =	{	10,  0,  8,  0, 11,  7,  9,  0,
											 0, 14, 16, 12,  0, 15,  0, 13};

	static BYTE MultiTableK6[8]  =		{	 9, 10,  8, 11,  5,  6, 12,  7 };
	static BYTE MultiTableK6M13[8]  =	{	 9, 10,  8, 11,  4,  6, 12,  7 };

	// 03/08/08 Athlon ̔{e[u͗v!!
	static BYTE MultiTableAthlon[32] = {	22, 23, 24, 25, 10, 11, 12, 13, 
											14, 15, 16, 17, 18, 19, 20, 21,
											50, 38, 52, 40, 26, 27, 28, 42, 
											30, 44, 32, 33, 34, 36, 46, 48};

	static BYTE MultiTableMobileAthlon[32]={22, 23, 24, 25, 10, 11, 12, 13, 
											14, 15, 16, 17, 18, 19, 20, 21,
											 6,  7,  8,  9, 26, 27, 28, 29, 
											30, 31, 32, 33, 34, 36, 38, 40};

	static BYTE MultiTableCoppermine[32]={	10,  6,  8,  4, 11,  7,  9,  5,
		     								 7, 14, 16, 12,  4, 15,  3, 13 ,
		     								18, 22, 24,  4, 19, 23, 17,  5,
		     								 7, 14, 16, 20,  4, 15,  3, 21 };

	static BYTE MultiTableTualatin[32]=	{	24,  7,  7,  8, 11,  7, 23,  7,
		     								32, 14, 16, 12,  8, 15,  7, 13 ,
		     								18, 22, 24,  7, 19,  7, 17,  7,
		     								 7,  7, 26, 20, 28,  7, 30, 21 };

	static BYTE MultiTableWillamette[16]={	32, 34, 36, 38, 40, 42, 44, 46,
											48, 50, 52, 54, 24, 26, 28, 30 };

	static BYTE MultiTableGeneric[16]=	{	10,  6,  8,  4, 11,  7,  9,  5,
		     								 7, 14, 16, 12,  4, 15,  3, 13 };

	if(FlagBrand == CYRIX || FlagBrand == IDT){
		bus = (MsrEAX1>>18)&0x3;
		mul = (MsrEAX1>>22)&0xF;
		switch (FlagMultiTable)
		{
		case Samuel2:	MultiplierOri = MultiTableSamuel2[mul] / 2.0;break;
		case Ezra:		MultiplierOri = MultiTableEzra[mul] / 2.0;break;
		case Cyrix3:	MultiplierOri = MultiTableCryix3[mul] / 2.0;break;
		default:
			if( (FlagBrand == IDT && Family == 5 && Model == 8 && 7 <= Stepping && Stepping <= 9) // WinChip 2A
			||	(FlagBrand == IDT && Family == 5 && Model == 9)// WinChip 3
			){
				MultiplierOri = ( ( (MsrEAX2 >> 23) & 0xF ) + 2.0) / ( (MsrEAX1 & 0x3) + 2.0 );
			}else if(FlagBrand == IDT && Model == 8){ // WinChip 2/B
				MultiplierOri = (MsrEAX1 & 0x3) + 2.0;
			}
			bus = 7;// `
			break;
		}
	}else if(FlagBrand == INTEL){
		bus = (MsrEAX1>>18)&0x3;
		mul = (MsrEAX1>>22)&0xF;

		switch (FlagMultiTable)
		{
		case Banias:
/*
			     if( strstr(NameString," 600") != 0){Multiplier = 6.0;}
			else if( strstr(NameString," 700") != 0){Multiplier = 7.0;}
			else if( strstr(NameString," 800") != 0){Multiplier = 8.0;}
			else if( strstr(NameString," 900") != 0){Multiplier = 9.0;}
			else if( strstr(NameString,"1000") != 0 || strstr(NameString,"1.0") != 0 ){Multiplier = 10.0;}
			else if( strstr(NameString,"1100") != 0 || strstr(NameString,"1.1") != 0 ){Multiplier = 11.0;}
			else if( strstr(NameString,"1200") != 0 || strstr(NameString,"1.2") != 0 ){Multiplier = 12.0;}
			else if( strstr(NameString,"1300") != 0 || strstr(NameString,"1.3") != 0 ){Multiplier = 13.0;}
			else if( strstr(NameString,"1400") != 0 || strstr(NameString,"1.4") != 0 ){Multiplier = 14.0;}
			else if( strstr(NameString,"1500") != 0 || strstr(NameString,"1.5") != 0 ){Multiplier = 15.0;}
			else if( strstr(NameString,"1600") != 0 || strstr(NameString,"1.6") != 0 ){Multiplier = 16.0;}
			else if( strstr(NameString,"1700") != 0 || strstr(NameString,"1.7") != 0 ){Multiplier = 17.0;}
			else if( strstr(NameString,"1800") != 0 || strstr(NameString,"1.8") != 0 ){Multiplier = 18.0;}
			else if( strstr(NameString,"1900") != 0 || strstr(NameString,"1.9") != 0 ){Multiplier = 19.0;}
			else if( strstr(NameString,"2000") != 0 || strstr(NameString,"2.0") != 0 ){Multiplier = 20.0;}
			else if( strstr(NameString,"2100") != 0 || strstr(NameString,"2.1") != 0 ){Multiplier = 21.0;}
			else if( strstr(NameString,"2200") != 0 || strstr(NameString,"2.2") != 0 ){Multiplier = 22.0;}
			else if( strstr(NameString,"2300") != 0 || strstr(NameString,"2.3") != 0 ){Multiplier = 23.0;}
			else if( strstr(NameString,"2400") != 0 || strstr(NameString,"2.4") != 0 ){Multiplier = 24.0;}
			else if( strstr(NameString,"2500") != 0 || strstr(NameString,"2.5") != 0 ){Multiplier = 25.0;}
*/
			Multiplier    = (MsrEDX2>>17)&0x1F;
			MultiplierOri = (MsrEAX1>>22)&0x1F;

			bus = 2;
			break;
		case Northwood:
			switch( (MsrEAX2>>16) & 0x7 )
			{
			case 0:	bus = 2;	break;	// 100MHz
			case 1:	bus = 1;	break;	// 133MHz
			case 2:	bus = 4;	break;	// 200MHz
			case 3:	bus = 3;	break;	// 166MHz
			}

			MultiplierOri = (MsrEAX2)&0x1F;
			Multiplier    = (MsrEAX2>>24)&0x1F;
			if(Multiplier < MultiplierOri){
				double temp = Multiplier;
				Multiplier = MultiplierOri;
				MultiplierOri = temp;
			}
			break;
		case Willamette:
			mul = (MsrEAX1>>8)&0xF;
			bus = 2;
			MultiplierOri = MultiTableWillamette[mul] / 2.0;
			break;
		case Coppermine:
			if( (MsrEAX1>>27)&0x1 && ( ( Model == 8 && Stepping >= 3 ) || Model == 0xA ) ){mul += 16;}
			MultiplierOri = MultiTableCoppermine[mul] / 2.0;
			break;
		case Tualatin:
			if( (MsrEAX1>>27)&0x1 ){mul += 16;}
			MultiplierOri = MultiTableTualatin[mul] / 2.0;break;
		case P6:
			if(Model <= 2 || (Model == 0x3 && Type == 0x1) ){bus = 7;} // `
			MultiplierOri = MultiTableGeneric[mul] / 2.0;
			break;
		}		
	}else if(FlagBrand == AMD){
		switch (FlagMultiTable)
		{
		case K6:
			mul = MsrEAX1&0xF;
			if(Model == 13){
				MultiplierOri = MultiTableK6M13[mul] / 2.0;
			}else{
				MultiplierOri = MultiTableK6[mul] / 2.0;
			}
			if(Model >= 0xD){
				bus = 2;
			}else{
				bus = 7;// `
			}
			break;
		case Athlon:
			mul = (MsrEAX1>>24)&0xF;
			if( (MsrEAX1>>19)&0x1 ){mul += 16;}

			if(CacheL2 == 64){bus = 2;}
			MultiplierOri = MultiTableAthlon[mul] / 2.0;
			SystemClock = Clock / MultiplierOri;

			if(Family == 0x6 && Model == 0xA){ //Barton or Thorton
				if( CacheL2 == 256 ){ // Thorton
					bus = 1; // 133MHz
				}else if( SystemClock >= 200.0 * 0.95 ){
					bus = 4; // 200MHz
				}else{
					bus = 3; // 166MHz
				}
			}else if(Family == 0x6 && Model == 0x8){//Ƃ肠 Thoroughbred
				if(CacheL2 == 64){ // Applebred
					bus = 1; // 133MHz
				}else if( (Stepping >= 1 && MultiplierOri >= 12.5 && SystemClock >= 166.66 * 0.95 ) ){
					bus = 3; // 166MHz
				}else{
					bus = 1; // 133MHz
				}
			}else if(Family == 0x6 && Model == 0x6){// Palomino
				bus = 1;
			}else{ // K7/K75
				bus = 2;
			}
			// Thunderbird ΍b
			if(Family == 0x6 && Model == 0x4){
				if(mul == 4){ MultiplierOri = 14.0; /* Thunderbird 1400MHz !? */	}
				if(mul == 3){ MultiplierOri = 13.0; /* Thunderbird 1300MHz */	}
				if( (int)(MultiplierOri*10) >= 110 || (int)(MultiplierOri*10) == 80 || (int)(MultiplierOri*10) == 95){
					bus = 2;//100
				}
				else if( (int)(MultiplierOri*10) == 105 ){bus = 1;}//133
				else{
					SystemClock = Clock / MultiplierOri;
					if(SystemClock > (133.33 * 0.95) ){bus = 1;} // 95%
					else{bus = 2;}
				}
			}
			break;
		case MobileAthlon:
			// PowerNOW! 
			// ( MsrEAX2 >> 16 ) & 0x1F
			Multiplier    = MultiTableMobileAthlon[ MsrEAX2 & 0x1F ] / 2.0;
			MultiplierOri = MultiTableMobileAthlon[ (MsrEAX2 >> 16) & 0x1F ] / 2.0;
			SystemClock = Clock / Multiplier;
			/*
			if(Multiplier < MultiplierOri){
				double temp = Multiplier;
				Multiplier = MultiplierOri;
				MultiplierOri = temp;
			}
			*/
			if(SystemClock > (133.33 * 0.95) ){bus=1;} // 95%
			else{bus = 2;}
			if(Model <= 7){bus = 2;} // Palomino/Morgan ͊m 100 MHz
			break;
		case Opteron:
			Multiplier = ( MsrEAX1 & 0x3F ) / 2.0 + 4.0;
			MultiplierOri = Multiplier;
			bus = 2;
			break;
		}
	}

	if(Multiplier < 0.0){ // ݂̔{`̂܂܂̏ꍇ...
		Multiplier = MultiplierOri;
	}

	// ςȔ{\邭炢Ȃ疢`
	if(Multiplier < 0.5){
		MultiplierOri = Multiplier = -1.0;
	}else{
		SystemClock = Clock / Multiplier;
		SystemBus = SystemClock * FSBMultiplier;
		SystemClockOri = BusTable[bus] / 100000.0;
		SystemBusOri = SystemClockOri * FSBMultiplier;
		ClockOri = SystemClockOri * MultiplierOri;
	}

	if(FlagBrand == TMx86){
		ClockOri = TmNominalClock;
	}
}

//////////////////////////////////////////////////////////////////////
// f[^̏
//////////////////////////////////////////////////////////////////////
void CCpuInfo::InitData()
{
	FlagCPUID = 0;
	FlagRDTSC = 0;
	FlagRDMSR = 0;
	FlagBrand = 0;
	FlagHalfSpeedCache = 0;
	FlagMultiTable = 0;
	FlagAMDMobile = 0;
	FlagClockModulation = 0;
	Clock = -1.0;
	Number = -1;
	Family = -1;
	Model = -1;
	Stepping = -1;
	FamilyEx = -1;
	ModelEx = -1;
	Type = -1;
	CacheL1I = -1;
	CacheL1T = -1;
	CacheL1U = -1;
	CacheL1D = -1;
	CacheL2  = -1;
	CacheL3  = -1;
	BrandID = 0;
	Feature = 0;
	FeatureEcx = 0;
	FeatureEx = 0;
	CacheSpeed = -1.0;
	OverClock = -100.0;
	Version = 0;
	VersionEx = 0;
	PlatformID = 0;
	Apic = -1;
	PhysicalNumber = -1;

	NameSysInfo[0] = '\0';
	FSBMode[0] = '\0';
	VendorString[0] = '\0';
	VendorName[0] = '\0';
	NameString[0] = '\0';
	PlatformName[0] = '\0';
	TypeName[0] = '\0';
	FMS[0] = '\0';
	CacheSpeedS[0] = '\0';
	Name[0] = '\0';
	CodeName[0] = '\0';
	FullName[0] = '\0';
	ProcessRule[0] = '\0';

// Transmeta
	TmNameString[0] = '\0';
	TmClock = -1;
	TmNominalClock = -1;

	SystemClock = -1.0;
	SystemBus = -1.0;
	Multiplier = -1.0;
	MultiplierOri = -1.0;

	ClockOri = -1.0;
	SystemClockOri = -1.0;
	SystemBusOri = -1.0;

	FSBMultiplier = -1;
	MsrEAX1 = 0;
	MsrEAX2 = 0;
	MsrEDX1 = 0;
	MsrEDX2 = 0;

	FlagMMX = 0;
	FlagMMXEx = 0;
	FlagSSE = 0;
	FlagSSE2 = 0;
	FlagSSE3 = 0;
	Flag3DNow = 0;
	Flag3DNowEx = 0;
	FlagHT = 0;
	FlagAA64 = 0;
	FlagIA64 = 0;

// CPU Number
	SYSTEM_INFO sysInfo;
	GetSystemInfo(&sysInfo);
	Number = sysInfo.dwNumberOfProcessors;

	L1DWaysS[0] = '\0';
	L1DLinesS[0] = '\0';
	L2WaysS[0] = '\0';
	L2LinesS[0] = '\0';
	L3WaysS[0] = '\0';
	L3LinesS[0] = '\0';
	L1ITUWaysS[0] = '\0';
	L1ITULinesS[0] = '\0';
	
	L1ITUWays = -1;
	L1ITULines = -1;
	L1DWays = -1;
	L1DLines = -1;
	L2Ways = -1;
	L2Lines = -1;
	L3Ways = -1;
	L3Lines = -1;

}

//////////////////////////////////////////////////////////////////////
// CPU  CPUID / RDTSC T|[gĂ邩𒲂ׂ
//////////////////////////////////////////////////////////////////////
int CCpuInfo::CheckUseCPUID()
{
	// CPUID߂̎gp
	DWORD flag_1, flag_2;
	_asm{
			pushfd
			pop		eax
			mov		flag_1, eax
			xor		eax, 00200000h
			push	eax
			popfd
			pushfd
			pop		eax
			mov		flag_2, eax
		}

    if( flag_1 == flag_2){	// CPUIDߎgps
		FlagCPUID = 0;
    }else{					// CPUIDߎgp
		FlagCPUID = 1;
	}

	return FlagCPUID;
}

//////////////////////////////////////////////////////////////////////
// Cache ̏擾
//////////////////////////////////////////////////////////////////////
int CCpuInfo::FillCacheInfo()
{
	if(CacheL1U > 0 || CacheL1I > 0 || CacheL1D > 0){
		return -1;
	}
	if(FlagBrand == INTEL){
		if(MaxCPUID >= 0x2){FillCacheInfoIntel();}
	}else{
		if(MaxCPUIDEx >= 0x80000005){FillCacheInfoAMD();}
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////
// Cache ̏擾 <INTEL>
//////////////////////////////////////////////////////////////////////
void CCpuInfo::FillCacheInfoIntel()
{
	DWORD dwEAX,dwEBX,dwECX,dwEDX;
	_asm{
		mov eax,2
		cpuid
		mov dwEAX,eax
		mov dwEBX,ebx
		mov dwECX,ecx
		mov dwEDX,edx
	}
	int loop = dwEAX & 0x000000FF;
	for(int i=1;i < loop;i++){
		_asm{
			mov eax,2
			cpuid
			mov dwEAX,eax
			mov dwEBX,ebx
			mov dwECX,ecx
			mov dwEDX,edx
		}
	}

	CacheInfoIntel( (dwEAX & 0xFF000000) >> 24 );
	CacheInfoIntel( (dwEAX & 0x00FF0000) >> 16 );
	CacheInfoIntel( (dwEAX & 0x0000FF00) >> 8  );
	CacheInfoIntel( (dwEBX & 0xFF000000) >> 24 );
	CacheInfoIntel( (dwEBX & 0x00FF0000) >> 16 );
	CacheInfoIntel( (dwEBX & 0x0000FF00) >> 8  );
	CacheInfoIntel(  dwEBX & 0x000000FF );
	CacheInfoIntel( (dwECX & 0xFF000000) >> 24 );
	CacheInfoIntel( (dwECX & 0x00FF0000) >> 16 );
	CacheInfoIntel( (dwECX & 0x0000FF00) >> 8  );
	CacheInfoIntel(  dwECX & 0x000000FF );
	CacheInfoIntel( (dwEDX & 0xFF000000) >> 24 );
	CacheInfoIntel( (dwEDX & 0x00FF0000) >> 16 );
	CacheInfoIntel( (dwEDX & 0x0000FF00) >> 8  );
	CacheInfoIntel(  dwEDX & 0x000000FF );

}
//////////////////////////////////////////////////////////////////////
// Cache ̏擾
//////////////////////////////////////////////////////////////////////
void CCpuInfo::CacheInfoIntel(int TEST)
{

	switch (TEST)
	{
	case 0x06:CacheL1I=8;L1ITUWays=4;L1ITULines=32;break;
	case 0x08:CacheL1I=16;L1ITUWays=4;L1ITULines=32;break;

	case 0x0A:CacheL1D=8;L1DWays=2;L1DLines=32;break;
	case 0x0C:CacheL1D=16;L1DWays=4;L1DLines=32;break;

	case 0x10:CacheL1D=16;L1DWays=4;L1DLines=32;break;		// IA-64
	case 0x15:CacheL1I=16;L1ITUWays=4;L1ITULines=32;break;	// IA-64
	case 0x1A:CacheL2=96;L2Ways=6;L2Lines=64;break;			// IA-64

	case 0x22:CacheL3=512;L3Ways=4;L3Lines=64;break;
	case 0x23:CacheL3=1024;L3Ways=8;L3Lines=64;break;
	case 0x25:CacheL3=2048;L3Ways=8;L3Lines=64;break;
	case 0x29:CacheL3=4096;L3Ways=8;L3Lines=64;break;

	case 0x2C:CacheL1D=32;L1DWays=8;L1DLines=64;break;
	case 0x30:CacheL1I=32;L1ITUWays=8;L1ITULines=64;break;

	case 0x39:CacheL2=128;L2Ways=4;L2Lines=64;break;
	case 0x3B:CacheL2=256;L2Ways=2;L2Lines=64;break;
	case 0x3C:CacheL2=256;L2Ways=4;L2Lines=64;break;

	case 0x40:
		if(Family == 6){CacheL2=0;}
		else{CacheL3=0;}		
		break;

	case 0x41:CacheL2=128;L2Ways=4;L2Lines=32;break;
	case 0x42:CacheL2=256;L2Ways=4;L2Lines=32;break;
	case 0x43:CacheL2=512;L2Ways=4;L2Lines=32;break;
	case 0x44:CacheL2=1024;L2Ways=4;L2Lines=32;break;
	case 0x45:CacheL2=2048;L2Ways=4;L2Lines=32;break;

	case 0x66:CacheL1D=8;L1DWays=4;L1DLines=64;break;
	case 0x67:CacheL1D=16;L1DWays=4;L1DLines=64;break;
	case 0x68:CacheL1D=32;L1DWays=4;L1DLines=64;break;

	case 0x70:CacheL1T=12;L1ITUWays=8;break;
	case 0x71:CacheL1T=16;L1ITUWays=8;break;
	case 0x72:CacheL1T=32;L1ITUWays=8;break;

	case 0x77:CacheL1I=16;L1ITUWays=8;L1ITULines=64;break;	// IA-64

	case 0x79:CacheL2=128;L2Ways=8;L2Lines=64;break;
	case 0x7A:CacheL2=256;L2Ways=8;L2Lines=64;break;
	case 0x7B:CacheL2=512;L2Ways=8;L2Lines=64;break;
	case 0x7C:CacheL2=1024;L2Ways=8;L2Lines=64;break;
	case 0x7E:CacheL2=256;L2Ways=8;L2Lines=128;break;

	case 0x81:CacheL2=128;L2Ways=8;L2Lines=32;break;
	case 0x82:CacheL2=256;L2Ways=8;L2Lines=32;break;
	case 0x83:CacheL2=512;L2Ways=8;L2Lines=32;break;
	case 0x84:CacheL2=1024;L2Ways=8;L2Lines=32;break;
	case 0x85:CacheL2=2048;L2Ways=8;L2Lines=32;break;

	case 0x86:CacheL2=512;L2Ways=8;L2Lines=64;break;
	case 0x87:CacheL2=1024;L2Ways=8;L2Lines=64;break;

	case 0x88:CacheL3=2048;L3Ways=4;L3Lines=64;break;		// IA-64
	case 0x89:CacheL3=4096;L3Ways=4;L3Lines=64;break;		// IA-64
	case 0x8A:CacheL3=8192;L3Ways=4;L3Lines=64;break;		// IA-64

	case 0x8D:CacheL3=3096;L3Ways=12;L3Lines=128;break;		// IA-64

	default :
		;
	}
	sprintf(L1ITUWaysS,"%d",L1ITUWays);
	sprintf(L1ITULinesS,"%d",L1ITULines);
	sprintf(L1DWaysS,"%d",L1DWays);
	sprintf(L1DLinesS,"%d",L1DLines);
	sprintf(L2WaysS,"%d",L2Ways);
	sprintf(L2LinesS,"%d",L2Lines);
	sprintf(L3WaysS,"%d",L3Ways);
	sprintf(L3LinesS,"%d",L3Lines);

}

//////////////////////////////////////////////////////////////////////
// Cache ̏擾 <AMD>
//////////////////////////////////////////////////////////////////////
void CCpuInfo::FillCacheInfoAMD()
{
	DWORD dwEAX,dwEBX,dwECX,dwEDX;
	_asm{
		mov eax,0x80000005
		cpuid
		mov dwEAX,eax
		mov dwEBX,ebx
		mov dwECX,ecx
		mov dwEDX,edx
	}
	CacheL1I = dwEDX >> 24;
	CacheL1D = dwECX >> 24;

	L1ITUWays = (dwEDX >> 16) & 0xFF;
	L1ITULines= dwEDX & 0xFF;
	L1DWays   = (dwECX >> 16) & 0xFF;
	L1DLines  = dwECX & 0xFF;

	sprintf(L1ITULinesS,"%d",L1ITULines);
	sprintf(L1DLinesS,"%d",L1DLines);

	if(L1ITUWays == 0xFF){
		sprintf(L1ITUWaysS,"Full");
	}else{
		sprintf(L1ITUWaysS,"%d",L1ITUWays);
	}
	if(L1DWays == 0xFF){
		sprintf(L1DWaysS,"Full");
	}else{
		sprintf(L1DWaysS,"%d",L1DWays);
	}

	if(FlagBrand == AMD && Family == 6 && Model == 3 && Stepping == 0){ // Duron Bug
		CacheL2 = 64;
	}else if(FlagBrand == AMD && Family == 6 && Model == 4 && Stepping <= 1){ // ThunderBird Bug
		CacheL2 = 256;
	}else if(MaxCPUIDEx >= 0x80000006){ // 񎟃LbV
		_asm{
			mov eax,0x80000006
			cpuid
			mov dwECX,ecx
		}
		CacheL2 = dwECX >> 16;
		if(FlagBrand == IDT && ( Family == 6 && Model >= 7 ) ){
			CacheL2 >>= 8;
		}else{
			L2Ways  = (dwECX >> 12) & 0xF;
			L2Lines = dwECX & 0xFF;

			sprintf(L2LinesS,"%d",L2Lines);

			switch(L2Ways){
				case 0x0:	sprintf(L2WaysS,"None");			break;
				case 0x1:	sprintf(L2WaysS,"Direct Mapped");	break;
				case 0x2:	sprintf(L2WaysS,"2");				break;
				case 0x4:	sprintf(L2WaysS,"4");				break;
				case 0x6:	sprintf(L2WaysS,"8");				break;
				case 0x8:	sprintf(L2WaysS,"16");				break;
				case 0xF:	sprintf(L2WaysS,"Full");			break;
				default:	sprintf(L2WaysS,"None");			break;
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////
// NameString ̏擾
//////////////////////////////////////////////////////////////////////
void CCpuInfo::FillNameString()
{
	char name[49];
	name[48]='\0';
	_asm{
		mov eax,0x80000002
		cpuid
		mov dword ptr [name],eax
		mov dword ptr [name+4],ebx
		mov dword ptr [name+8],ecx
		mov dword ptr [name+12],edx
		mov eax,0x80000003
		cpuid
		mov dword ptr [name+16],eax
		mov dword ptr [name+20],ebx
		mov dword ptr [name+24],ecx
		mov dword ptr [name+28],edx
		mov eax,0x80000004
		cpuid
		mov dword ptr [name+32],eax
		mov dword ptr [name+36],ebx
		mov dword ptr [name+40],ecx
		mov dword ptr [name+44],edx
	}
	
	// 03/7/12
	// ÕXy[XXLbv
	char *p;
	p = name;
	while(*p == ' '){p++;}

	sprintf(NameString,p);
}

//////////////////////////////////////////////////////////////////////
// Transmeta pg擾
//////////////////////////////////////////////////////////////////////
void CCpuInfo::FillTMx86Ex()
{
	DWORD Max8086;
	DWORD dwEAX,dwECX;

	_asm{
		mov eax,0x80860000
		cpuid
		mov Max8086,eax
	}

	// WNbNg //
	if(Max8086 >= 0x80860001){
		_asm{
			mov eax,0x80860001
			cpuid
			mov dwECX,ecx
		}
	}
	TmNominalClock = dwECX;

	char name[65];name[64]='\0';
	if(Max8086 >= 0x80860006){
		_asm{
			mov eax,0x80860003
			cpuid
			mov dword ptr [name],eax
			mov dword ptr [name+4],ebx
			mov dword ptr [name+8],ecx
			mov dword ptr [name+12],edx
			mov eax,0x80860004
			cpuid
			mov dword ptr [name+16],eax
			mov dword ptr [name+20],ebx
			mov dword ptr [name+24],ecx
			mov dword ptr [name+28],edx
			mov eax,0x80860005
			cpuid
			mov dword ptr [name+32],eax
			mov dword ptr [name+36],ebx
			mov dword ptr [name+40],ecx
			mov dword ptr [name+44],edx
			mov eax,0x80860006
			cpuid
			mov dword ptr [name+48],eax
			mov dword ptr [name+52],ebx
			mov dword ptr [name+56],ecx
			mov dword ptr [name+60],edx
		}
		sprintf(TmNameString,name);
	}

	// ݂̃NbNg // 
	if(Max8086 >= 0x80860007){
		_asm{
			mov eax,0x80860007
			cpuid
			mov dwEAX,eax
		}
	}
	TmClock = dwEAX;
}
